@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
 
-/* $Id: pcsc_scan.c,v 1.16 2005/04/19 19:20:08 rousseau Exp $ */
+/* $Id: pcsc_scan.c,v 1.21 2006-03-09 13:48:19 rousseau Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,9 +64,13 @@ int main(int argc, char *argv[])
 	char atr_command[sizeof(atr)+sizeof(ATR_PARSER)+2+1];
 	int opt;
 	int analyse_atr = TRUE;
+	char *blue = "";
+	char *red = "";
+	char *magenta = "";
+	char *color_end = "";
 
 	printf("PC/SC device scanner\n");
-	printf("V " VERSION " (c) 2001-2004, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
+	printf("V " VERSION " (c) 2001-2006, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
 	printf("Compiled with PC/SC lite version: " PCSCLITE_VERSION_NUMBER "\n");
 
 	while ((opt = getopt(argc, argv, "Vhn")) != EOF)
@@ -96,6 +100,33 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	/* check if terminal supports color */
+	{
+		const char *terms[] = { "linux", "xterm", "xterm-color", "Eterm", "rxvt", "rxvt-unicode" };
+		char *term;
+
+		term = getenv("TERM");
+		if (term)
+		{
+			int i;
+
+			/* for each known color terminal */
+			for (i = 0; i < sizeof(terms) / sizeof(terms[0]); i++)
+			{
+				/* we found a supported term? */
+				if (0 == strcmp(terms[i], term))
+				{
+					blue = "\33[34m";
+					red = "\33[31m";
+					magenta = "\33[35m";
+					color_end = "\33[0m";
+					break;
+				}
+			}
+		}
+
+	}
+
 	rv = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
 	if (rv != SCARD_S_SUCCESS)
 	{
@@ -105,10 +136,17 @@ int main(int argc, char *argv[])
 
 get_readers:
 	/* free memory possibly allocated in a previous loop */
-	if (NULL == readers)
+	if (NULL != readers)
+	{
 		free(readers);
-	if (NULL == rgReaderStates_t)
+		readers = NULL;
+	}
+
+	if (NULL != rgReaderStates_t)
+	{
 		free(rgReaderStates_t);
+		rgReaderStates_t = NULL;
+	}
 
 	/* Retrieve the available readers list.
 	 *
@@ -116,7 +154,7 @@ get_readers:
 	 * 2. malloc the necessary storage
 	 * 3. call with the real allocated buffer
 	 */
-	printf("Scanning present readers\n");
+	printf("%sScanning present readers%s\n", magenta, color_end);
 	rv = SCardListReaders(hContext, NULL, NULL, &dwReaders);
 	if (rv != SCARD_S_SUCCESS)
 	{
@@ -126,7 +164,10 @@ get_readers:
 
 	/* if non NULL we came back so free first */
 	if (mszReaders)
+	{
 		free(mszReaders);
+		mszReaders = NULL;
+	}
 
 	mszReaders = malloc(sizeof(char)*dwReaders);
 	if (mszReaders == NULL)
@@ -153,7 +194,7 @@ get_readers:
 
 	if (nbReaders == 0)
 	{
-		printf("Waiting for the first reader...");
+		printf("%sWaiting for the first reader...%s", red, color_end);
 		fflush(stdout);
 		while ((SCardListReaders(hContext, NULL, NULL, &dwReaders)
 			== SCARD_S_SUCCESS) && (dwReaders == dwReadersOld))
@@ -175,7 +216,7 @@ get_readers:
 	ptr = mszReaders;
 	while (*ptr != '\0')
 	{
-		printf("%d: %s\n", nbReaders, ptr);
+		printf("%s%d: %s%s\n", blue, nbReaders, ptr, color_end);
 		readers[nbReaders] = ptr;
 		ptr += strlen(ptr)+1;
 		nbReaders++;
@@ -236,11 +277,12 @@ get_readers:
 			t = time(NULL);
 
 			/* Specify the current reader's number and name */
-			printf("\n%s Reader %d (%s)\n", ctime(&t), current_reader,
-				rgReaderStates_t[current_reader].szReader);
+			printf("\n%s Reader %d: %s%s%s\n", ctime(&t), current_reader,
+				magenta, rgReaderStates_t[current_reader].szReader,
+				color_end);
 
 			/* Dump the full current state */
-			printf("\tCard state: ");
+			printf("  Card state: %s", magenta);
 
 			if (rgReaderStates_t[current_reader].dwEventState &
 				SCARD_STATE_IGNORE)
@@ -281,12 +323,12 @@ get_readers:
 				SCARD_STATE_MUTE)
 				printf("Unresponsive card, ");
 
-			printf("\n");
+			printf("%s\n", color_end);
 
 			/* Also dump the ATR if available */
 			if (rgReaderStates_t[current_reader].cbAtr > 0)
 			{
-				printf("\tATR: ");
+				printf("  ATR: ");
 
 				if (rgReaderStates_t[current_reader].cbAtr)
 				{
@@ -299,7 +341,7 @@ get_readers:
 				else
 					atr[0] = '\0';
 
-				printf("%s\n\n", atr);
+				printf("%s%s%s\n\n", magenta, atr, color_end);
 
 				if (analyse_atr)
 				{
@@ -325,9 +367,9 @@ get_readers:
 	}
 
 	/* free memory possibly allocated */
-	if (NULL == readers)
+	if (NULL != readers)
 		free(readers);
-	if (NULL == rgReaderStates_t)
+	if (NULL != rgReaderStates_t)
 		free(rgReaderStates_t);
 
 	return 0;
